;
; misc.asm - misc stuff
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;------------------------------
;Used by #devicehandler macro
;------------------------------
		#opencfg_start
set_current_drive_from_drive_number .proc
		tay
		ldx #0
cdrvlp		cmp current_drivenumbers_exp,x
		beq cdrvfnd
		inx
                cpx #current_drivenumbers_end_exp-current_drivenumbers_exp
		bcc cdrvlp
cdrvunk		ldx #DRIVES+2		;DRIVES+2 unknown
cdrvok		.if SUPERCPU=1
		txa
		asl
		tax
		.fi
		rts

cdrvfnd		txa
		cmp #DRIVES+2		;>DRIVES+2 floppy 8-11
		bcc cdrvnf
		adc #7-DRIVES-2
		tay
		gne cdrvunk

cdrvnf		cmp #DRIVES		;DRIVES = pc DRIVES+1 = lcd
		bcs cdrvok
		sta current_drive_exp
		lda drive_type_exp,x	;Not existing IDE drive
		beq cdrvunk
		ldx #DRIVES-1		;DRIVES-1 = ide
		gcc cdrvok
		.pend
		#opencfg_end

exit_too_many_files .proc
		lda #1
		.byte SKIP2
		.pend
exit_file_open	.proc
		lda #2
		.byte SKIP2
		.pend
exit_file_not_open .proc
		lda #3
		.byte SKIP2
		.pend
exit_file_not_found .proc
		lda #4
		.byte SKIP2
		.pend
exit_device_not_present .proc
		lda #5
		.byte SKIP2
		.pend
exit_not_input_file .proc
		lda #6
		.byte SKIP2
		.pend
exit_not_output_file .proc
		lda #7
		.byte SKIP2
		.pend
exit_missing_filename .proc
		lda #8
		.byte SKIP2
		.pend
exit_illegal_device .proc
		lda #9
		.byte SKIP2
		.pend
exit_file_data	.proc
		lda #$18
		.byte SKIP2
		.pend
exit_out_of_memory
		lda #$10
		pha
		#psr $f716
		jmp exit2basic
;------------------------------
;Guess what ;) (usefull for clock reading)
;------------------------------
bcd2bin		.proc
		pha
		lsr
		lsr
		lsr
		lsr
		tax
		pla
		cpx #0
		beq iybmhdy
jkq		sbc #6
		dex
		bne jkq
iybmhdy		rts
		.pend

newchrgot	.proc
		dec $1
		jsr $79
		php
		inc $1
		plp
		rts
		.pend

newchrget	.proc
		dec $1
		jsr $73
		php
		inc $1
		plp
		rts
		.pend

OP_cfg16up	.proc			;Go to cfg_16, but only replace $a000-$bfff
		lda $01
		sta help_01_exp
		lda #$36
		sta $01
		sta cfg_16
		rts
		.pend

OP_cfg16down	.proc			;Come back from replacing $a000-$bfff
		sta cfg_open
		lda help_01_exp
		sta $01
		rts
		.pend

		.if SUPERCPU=1
OP_scpu01up	.proc
		lda $01
		sta help_01_scpu_exp
		ora #7
		sta $01
		rts
		.pend

OP_scpu01down	.proc
		lda help_01_scpu_exp
		sta $01
		rts
		.pend
		.fi

                .if SUPERCPU=1
scpu_default_opt .proc
		php
                sei
		sta scpu_hwenable
                sta scpu_nooptim	;default optimization settings
                lda #%11000001
                sta scpu_optimv2
                sta scpu_hwdisable
                plp
                rts
                .pend
                .fi

                .if SUPERCPU=1
scpu_no_opt	.proc
		php
                sei
                sta scpu_hwenable
                sta scpu_nooptim	;no optimization, sprites!!!
                sta scpu_hwdisable
                plp
                rts
                .pend
                .fi

		.if SERIAL_HANG_FIX=1
test_hang	pha
		#phx_a
		ldx #0
		lda $dd00
		and #$df
		sta $dd00
mn		lda $dd00
		asl
		bcs vr
		lda $d011
		bpl *-3
		lda $d011
		bmi *-3
		dex
		bne mn
		lda #$80
		sta Status
vr		#plx_a
		pla
		rts
		.fi
;----------------------------------
;Print hex number
;
NO_printhx	.proc
		pha
		lsr
		lsr
		lsr
		lsr
		jsr ijou
		pla
		and #15
ijou		cmp #10
		bcc *+4
		adc #6
		adc #$30
		jmp newchrout_t
		.pend
;----------------------------------
;Check RUN/STOP Z=1 if pressed
;
check_stop	.proc
		lda StopFlag
		eor #$7f
		bne *+4
		sta $c6
		rts
		.pend
;---------------------------------------
;Print imediate
;---------------------------------------
LWNO_print_imediate .proc
		pla
		sta $22
		pla
		jsr printall
                #psz $22
		rts
		.pend
;---------------------------------------
;Print A/Y
;---------------------------------------
LWNO_print_ay	.proc
		sta $22
		sty $23
		ldy #0
		geq printall.cbv
;---------------------------------------
;Printing common
;---------------------------------------
printall	.proc
		sta $23
		sty $24
		ldy #0
svc		inc $22
		bne cbv
		inc $23
cbv		lda ($22),y
		beq prntki
		jsr newchrout_t		; chrout
		gcc svc
prntki		ldy $24
		rts
		.pend
;--------------------------------------
;make drive number to default
;--------------------------------------
OP_hdinit_drivenumbers .proc
		ldx #(current_drivenumbers_end_exp-current_drivenumbers_exp)/2
lp		lda setup_drivenumbers_exp-1,x
		pha
                and #$0f
                beq jq
                cpx #DRIVES+1
                bne nq
                sbc #4
nq              clc
                adc #7
jq		sta current_drivenumbers_exp-1,x
                pla
                lsr
                lsr
                lsr
                lsr
                beq jq2
                clc
                adc #7
jq2             sta current_drivenumbers_exp-1+(current_drivenumbers_end_exp-current_drivenumbers_exp)/2,x
		dex
		bne lp
		lda setup_boot_drive_exp
		sta ActualDev
		rts
		.pend

buffer_subpart_cp .proc
		#ldx_l_ buffer_currentp_exp
buffer_subpart_cp2 .proc
		ora buffer_start,x
		sta current_path+1
		.if SUPERCPU=1
		ldy #1
		sty current_path+2
		dey
		.else
		ldy #0
		.fi
		sty current_path
		rts
		.pend

buffer_tree	.proc
		asl
		sta pom1
		lda buffer_start,x
		adc #>b_cfs_file_tree_exp
		sta pom2
		.if SUPERCPU=1
		lda #1
		sta pom3
		.fi
		rts
		.pend

;-------------------------
;Save or restore allocator state
;
save_allocator_state .proc
		sec
		.byte SKIP1
restore_allocator_state .proc
		clc
		php
		lda #>b_cfs_file_misc_exp
		jsr buffer_subpart_cp
		ldy #<(b_cfs_file_free_exp+find_free_backup_end_exp-find_free_backup_start_exp-1)
		ldx #find_free_backup_end_exp-find_free_backup_start_exp-1
		plp
cwe		lda find_free_backup_start_exp,x
		bcs vs
		#lda_zp_y current_path
vs		#sta_zp_y current_path
		sta find_free_backup_start_exp,x
		dey			;restore allocator state
		dex
		bpl cwe
		rts
		.pend

OP_loadautos	.proc
		lda load_autostart_exp	;no autostart
		sta load_autostart2_exp	;no autostart
		#lda0
		sta load_autostart_exp	;no autostart
		rts
		.pend
